/**
 * 小程序
 * 首页基础数据
 */

import * as partyMemberData from '../../data/partyMember';
import * as partyInformationData from '../../data/partyInformation';
import * as configEnum from "../../config/enum";
import * as splitResultConfig from "../../config/splitResultConfig";
import { extractData } from '../../util/piecemeal';
import { changeEnumValue, eccEnumValue } from '../../util/verificationEnum';
import { BizError } from '../../util/bizError';
import { ERRORENUM } from '../../config/errorEnum';


/**
 * 主页基础数据
 */
export async function homeData() {
    /**党员总数 */
    let partyMemberNum = await partyMemberData.groupFindPartyMemberCount({});
    /**党建指令指数 */
    let massIndex = 87.6;

    /**党内职务人员 */ 
    let partyPositions = [];
    for (let key in configEnum.PARTYPOSITIONS) {
        let anyKey:any = key;
        if (isNaN(anyKey)) { //枚举key  key = string
            const EnumValue = parseInt(configEnum.PARTYPOSITIONS[key])
            let dataBaseList = await partyMemberData.getBaseListByPartyPositions( EnumValue );
    
            let nameList = [];
            dataBaseList.forEach( info => {
                nameList.push(info.name);
            });

            partyPositions.push({
                key,
                list:nameList
            });

        }
    }

    return { partyMemberNum, massIndex, partyPositions };
}


/**
 * 根据资讯类型查询党建资讯列表
 * @param informationType 党建资讯类型
 * @returns 
 */
export async function informationData(informationType:number) {
    let selectParam:any = {};
    if (informationType) {
        eccEnumValue("党建资讯列表查询", "资讯类型", configEnum.INFORMATIONTYPE, informationType);
        selectParam.informationType = informationType;
    }

    /**医院文化 */
    let partyInformationInfo = await partyInformationData.findPartyInformationList(selectParam);

    let dataList = [];
    partyInformationInfo.forEach( info => {
        let changeData:any = extractData(splitResultConfig.partyInformationConfig, info, false);
        changeData.informationType = changeEnumValue(configEnum.INFORMATIONTYPE, changeData.informationType);

        dataList.push(changeData);
    })

    return dataList;
}



