/**
 * 小程序
 * 组织生活
 */

import moment = require("moment");
import * as organizationalLifeData from "../../../data/organizationalLife";
import * as configEnum from "../../../config/enum";
import { eccFormParam } from "../../../util/verificationParam";
import * as eccFormParamConfig from "../../../config/eccFormParamConfig";
import * as verificationEnumTools from "../../../util/verificationEnum";
import { ERRORENUM } from "../../../config/errorEnum";
import { BizError } from "../../../util/bizError";
import { extractData } from "../../../util/piecemeal";
import * as splitResultConfig from "../../../config/splitResultConfig";


/**
 * 组织生活列表
 * todo  接口协议不对
 * @param status 活动状态 全部、正在进行、即将开展、已经开展
 * @return {allActivity, afootActivity, beginInActivity, startedActivity}
 * 返回说明 {全部活动列表, 正在进行活动列表, 即将开展活动列表, 已经开展活动列表}
 */
export async function getOrganizationalLifeList(type) {

    let selectParam = {};
    let now = new Date().valueOf();
    switch (type) {
        case enum.正在进行:
            selectParam.startTime:{"$gt"}
    }

    1000

    let organizationalLifeInfo = await organizationalLifeData.findOrganizationalLifeList(selectParam);

    
    /**全部活动列表 */
    let allActivity = [];
    /**正在进行活动列表 */
    let afootActivity = [];
    /**即将开展活动列表 */
    let beginInActivity = [];
    /**已经开展活动列表 */
    let startedActivity = [];

    organizationalLifeInfo.forEach( info => {
        let activityStatus = getActivityStatus(info.activityStartTime, info.activityEndTime)
        let activityTime = "2023.05.17 14:00--16:00"; // -----------todo 活动时间 格式不确定，之后修改---------------------
        
        allActivity.push({
            _id: info._id,
            partyActivityType: info.partyActivityType,
            activityTime,
            activityAdd: info.activityAdd,
            activityStatus
        });

        switch (activityStatus) {
            case "正在进行":
                afootActivity.push({
                    _id: info._id,
                    partyActivityType: info.partyActivityType,
                    activityTime,
                    activityAdd: info.activityAdd,
                    activityStatus
                });
            break;
            case "即将开展":
                beginInActivity.push({
                    _id: info._id,
                    partyActivityType: info.partyActivityType,
                    activityTime,
                    activityAdd: info.activityAdd,
                    activityStatus
                });
            break;
            case "已经开展":
                startedActivity.push({
                    _id: info._id,
                    partyActivityType: info.partyActivityType,
                    activityTime,
                    activityAdd: info.activityAdd,
                    activityStatus
                });
            break;
        }
    })

    return {allActivity, afootActivity, beginInActivity, startedActivity};
}


/**
 * 组织生活活动状态
 * 活动状态 1：正在进行、2：即将开展、3：已经开展
 * @param activityStartTime 活动开始时间
 * @param activityEndTime 活动结束时间
 * @return status --活动状态
 */
function getActivityStatus(activityStartTime, activityEndTime) {
    let nowTime = new Date().valueOf();

    let startTime = moment(activityStartTime).format("YYYY.MM.DD HH:mm");
    let endTime = moment(activityEndTime).format("YYYY.MM.DD HH:mm");
    let underway = moment().isBetween(startTime, endTime); //true：正在进行

    let status;
    if (underway) {
        status = "正在进行";
    }

    if (nowTime < activityStartTime) {
        status = "即将开展";
    }

    if (nowTime > activityStartTime) {
        status = "已经开展";
    }

    return status
}


/**
 * 创建组织生活活动
 * @param param  {党建活动类型, 活动开始时间, 活动结束时间, 活动地点, 活动主持人, 活动记录人, 请假党员, 参与党员, 活动记录}
 *      参数类型  {  number,      number,      number,    string,  [string],  [string],  [string], [string], string}
 * @return {isSuccess: true}
 */
export async function createActivity(param) {
    eccFormParam("创建组织生活活动", eccFormParamConfig.CreateActivityConfig, param)
    verificationEnumTools.eccEnumValue('创建组织生活活动的党建活动类型', 'partyActivityType', configEnum.ACTIVITYTYPE, param.partyActivityType);

    await organizationalLifeData.createActivity(param);

    return {isSuccess: true};
}


export async function getActivityById(_id) {
    let activityInfo = await organizationalLifeData.findEventDetails(_id);
    if (!activityInfo ) throw new BizError(ERRORENUM.未找到数据, `库中不存在${_id}这个组织生活活动`);

    /**截取返回字段 */
    let data = extractData(splitResultConfig.activityInfoConfig, activityInfo, false);


}