/**
 * 党员基本信息表
 */

import { Schema } from "mongoose";
import {yfsDB} from '../db/mongo/dbInit';

const partyMemberSchema = new Schema({
    _id: {type: String, index:true},
    /**姓名 必填 */
    name: String,
    /**性别 0：男、1：女 必填 */
    sex: Number,
    /**出生年月 必填 */
    birthDate: Number,
    /**手机号码 必填 */
    phone: Number,
    /**文化程度 必填 */
    levelOfEducation: Number,
    /**入党时间 必填 */
    partyJoiningTime: Number,
    /**行政职务 必填 1： */
    administrativePosition: Number,
    /**备注 非必填 */
    remarks: String,
    /**名册录入类型 1：正式党员录入、2：入党积极分子录入、3：入党申请录入 */
    entryType: Number,
    /**党内职务 多选 --正式党员录入  1：书记、2：副书记、3：纪检委员、4：组织委员、5：宣传委员 */
    partyPositions: [Number],
    /**列为积极份子时间 --入党积极份子录入 */
    listedAsActivistsTime: Number,
    /**谈话时间 --入党申请录入 */
    talkTime: Number,

    /**-----用户相关----- */
    /**登录账号 */
    loginId: {type: String, index: true},
    /**密码 */
    pwd: String,
    token:{type:String, index:true},
    tokenMs:Number,
    /**首次登录是否修改密码 */
    firstLoginIsChangePwd:{type:Boolean, default:false},
})

var partyMemberModel;

export function initModel(){
    partyMemberModel = yfsDB.model('partyMember', partyMemberSchema);
    partyMemberModel.selectOnceData = async function (paramater:object) {
        let selectInfo = await partyMemberModel.findOne(paramater).exec();
        if (selectInfo) {
            if (!selectInfo.runSave) {
                selectInfo.runSave = selectInfo.save;
                selectInfo.save = save.bind(selectInfo)
            }
        }
        return selectInfo;
    }
}

export async function save(throwError=false) {
    if (!this.isModified()) return;
    await this.runSave({validateBeforeSave:false}).catch(err=>{
        console.log(err);
    });
}

export async function findPartyMemberAll() {
    return await partyMemberModel.find({});
}


/**
 * 通过登录账号查询党员信息
 * @param loginId 登录账号
 * @returns 
 */
export async function findPartyMemberByLoginId(loginId:string) {
    return await partyMemberModel.selectOnceData({loginId});
}


/**
 * 获取符合条件的党员数量
 * @param selectParam 查询参数
 * @returns number 数据数量
 */
export async function groupFindPartyMemberCount(selectParam) {
    return await partyMemberModel.find(selectParam).count();
}


/**
 * 根据党内职务查询在职人员信息
 * @param partyPositions  党内职务
 * @returns 
 */
export async function getBaseListByPartyPositions(partyPositions:number) {
    //todo  sql有问题，重写一下
    return await partyMemberModel.find({partyPositions:{"in": partyPositions }});
}


